import { useContext } from 'react';
import { useQuery } from '@apollo/client';
import type { OperationVariables, QueryResult, QueryHookOptions, DocumentNode, ApolloError, FetchMoreQueryOptions, ApolloQueryResult } from '@apollo/client';

import { NotificationContext } from '@teambit/ui-foundation.ui.notifications.notification-context';

import { useLoader } from '@teambit/ui-foundation.ui.global-loader';

export type DataQueryResult<TData = any, TVariables = OperationVariables> = Omit<QueryResult<TData, TVariables>, 'data' | 'previousData' | 'fetchMore' | 'refetch'> & {
  data?: TData | undefined;
  previousData?: TData | undefined;
  fetchMore: (options: FetchMoreQueryOptions<TVariables, TData>) => Promise<ApolloQueryResult<TData>>;
  refetch: (variables?: Partial<TVariables>) => Promise<ApolloQueryResult<TData>>;
};

export function useDataQuery<TData = any, TVariables = OperationVariables>(
  query: DocumentNode,
  options?: QueryHookOptions<TData, TVariables>
): DataQueryResult {
  const res = useQuery<TData, TVariables>(query, options);
  const notifications = useContext(NotificationContext);

  const { loading, error } = res;
  useLoader(loading);

  if (error) {
    notifications.error(apolloErrorToString(error));
  }

  return res as DataQueryResult;
}

// @TODO - improve error extraction
function apolloErrorToString(error: ApolloError) {
  return error.toString();
}
